@@ -11,8 +11,8 @@ import re
 import os
 import libxml2
 
-import utils # TODO: split this modul
-import bughelper_error as Error
+import exceptions
+from exceptions import parse_error
 
 from bugbase import Bug as BugBase
 from lptime import LPTime
@@ -20,15 +20,9 @@ from tasksbase import LPTasks, LPTask
 from attachmentsbase import LPAttachments, LPAttachment
 from commentsbase import LPComments, LPComment
 from lphelper import user, product, change_obj, unicode_for_libxml2
-from lpconstants import BUG
+from lpconstants import BUG, BASEURL
 from subscribersbase import LPSubscribers
-
-
-DEBUG = True
-DEBUG_URL = False
-def debug(*arg,**args):
-    if DEBUG:
-        return utils.debug(*arg,**args)
+from utils import valid_lp_url
         
 # deactivate error messages from the validation [libxml2.htmlParseDoc]
 def noerr(ctx, str):
@@ -111,7 +105,24 @@ def _gen_setter(attr):
         setattr(reduce(getattr, attributes[:-1]), attributes[-1], value)
     return func
 
+def create_project_task(project):
+    x = [None]*14
+    task = Info(product(project), *x)
+    task._type = "%s.product" %project
+    return task
 
+def create_distro_task(distro=None, sourcepackage=None, url=None):
+    x = [None]*14
+    if distro is None:
+        distro = "Ubuntu"
+    if not sourcepackage is None:
+        affects = product("%s (%s)"%(sourcepackage, distro.title()))
+    else:
+        affects = product(distro.title())
+    task = Info(affects, *x)
+    task._type = "%s.sourcepackagename" %affects
+    task._remote = url
+    return task
         
 class Info(LPTask):
     """ The 'Info'-object represents one row of the 'InfoTable' of a bugreport
@@ -232,8 +243,9 @@ class Info(LPTask):
             args['%s.assignee' %full_sourcepackage] = self.assignee or ""
                 
             result = self._connection.post(self._editurl, args)
-            if result.url == self._editurl and not ignore_lp_errors:
-                raise ValueError, "There is a problem with the information you entered. Please fix it and try again:\n%s\n%s" % (self._editurl, repr(args))
+            if result.url.endswith("+editstatus") and not ignore_lp_errors:
+                raise exceptions.PythonLaunchpadBugsValueError({"arguments": args},
+                    self._editurl, "one or more arguments might be wrong")
 
 
 class InfoTable(LPTasks):
@@ -249,6 +261,8 @@ class InfoTable(LPTasks):
     """
     def __init__(self, connection, xml, url):
         LPTasks.__init__(self, {"connection":connection, "url":url, "xml":xml})
+        self.__url = url
+        self.__xml = xml
 
     def parse(self):
         """ Parsing the info-table
@@ -261,8 +275,9 @@ class InfoTable(LPTasks):
         """
         if self.parsed:
             return True
-        rows = self._xml[0].xpathEval('tbody/tr[not(@style="display: none") and not(@class="secondary")]')
-        assert rows, "Wrong XPath-Expr in InfoTable.parse() 'rows' (%s)" %(DEBUG_URL or "unknown")
+        rows = self.__xml[0].xpathEval('tbody/tr[not(@style="display: none") and not(@class="secondary")]')
+        parse_error(self.__xml[0], "InfoTable.rows", xml=self.__xml, url=self.__url)
+        
         highl_target = None
         # status of (remote)-bugs can be 'unknown''
         temp_status = BUG.STATUS.copy()
@@ -277,7 +292,7 @@ class InfoTable(LPTasks):
             edit_fields = set()
             tmp_affects = affects
             current = False
-            remote = False
+            remote = None
                 
             if row.prop("class") == "highlight":
                 current = True
@@ -287,12 +302,14 @@ class InfoTable(LPTasks):
             
             if not tracked:
                 # parse affects
-                assert row_cells[0], "Wrong XPath-Expr in InfoTable.parse() 'affects' (%s)" %(DEBUG_URL or "unknown")
+                parse_error(row_cells[0], "InfoTable.affects.1", xml=row_cells, url=self.__url)
                 m_type = row_cells[0].xpathEval("img")
-                assert m_type, "Wrong XPath-Expr in InfoTable.parse() 'affects' (%s)" %(DEBUG_URL or "unknown")
+                parse_error(m_type, "InfoTable.affects.2", xml=row_cells, url=self.__url)
+                
                 affects_type = m_type[0].prop("src").split("/")[-1]
                 m_name = row_cells[0].xpathEval("a")
-                assert m_name, "Wrong XPath-Expr in InfoTable.parse() 'affects' (%s)" %(DEBUG_URL or "unknown")
+                parse_error(m_name, "InfoTable.affects.3", xml=row_cells, url=self.__url)
+                
                 affects_lpname = m_name[0].prop("href").split("/")[-1]
                 affects_longname = m_name[0].content
             affects = product(affects_lpname, affects_longname, affects_type)
@@ -325,11 +342,17 @@ class InfoTable(LPTasks):
             if xmledit:
                 xmledit = xmledit[0]
                 editurl = xmledit.xpathEval('td/form')
-                assert editurl, "Wrong XPath-Expr in InfoTable.parse() 'editurl' (%s)" %(DEBUG_URL or "unknown")
-                editurl = utils.valid_lp_url(editurl[0].prop('action'), utils.BUG)
+                parse_error(editurl, "InfoTable.editurl", xml=xmledit, url=self.__url)
+                
+                editurl = valid_lp_url(editurl[0].prop('action'), BASEURL.BUG)
             
                 if xmledit.xpathEval('descendant::label[contains(@for,"bugwatch")]'):
-                    remote = True                
+                    x = xmledit.xpathEval('descendant::label[contains(@for,"bugwatch")]/a')
+                    if x:
+                        remote = x[0].prop("href")
+                    else:
+                        # if the status of the bug is updated manually.
+                        remote = True
                     
                 if not remote:
                     for i in ["product", "sourcepackagename"]:
@@ -338,7 +361,7 @@ class InfoTable(LPTasks):
                             type = x[0].prop("id")
                     if not type:
                         if not row.xpathEval('td[2]//img[contains(@src,"milestone")]'):
-                            assert type, "Wrong XPath-Expr in InfoTable.parse() 'type' (%s)" %(DEBUG_URL or "unknown")
+                            parse_error(False, "InfoTable.type.milestone", xml=xmledit, url=self.__url)
                 
                     m = xmledit.xpathEval('descendant::select[contains(@id,".milestone")]//option')
                     if m:
@@ -347,7 +370,7 @@ class InfoTable(LPTasks):
                             if i.prop("selected"):
                                 milestone = i.content
                             
-                    m = xmledit.xpathEval('descendant::td[@title="Changeable only by a project maintainer or bug contact" and count(span)=0]')
+                    m = xmledit.xpathEval('descendant::td[contains(@title, "Changeable only by a project maintainer") and count(span)=0]')
                     if len(m) == 1 and not milestone:
                         milestone = m[0].content.strip("\n ")
             
@@ -364,12 +387,18 @@ class InfoTable(LPTasks):
             else:
                 editlock = True
                         
-            assert row_cells[1], "Wrong XPath-Expr in InfoTable.parse() 'status' (%s)" %(DEBUG_URL or "unknown")
-            assert row_cells[1].prop("class") in temp_status, "unknown bugstatus '%s' in InfoTable.parse() (%s)" %(row_cells[1].prop("class"), DEBUG_URL or "unknown")
+            parse_error(row_cells[1], "InfoTable.status.1", xml=row_cells, url=self.__url)
+            parse_error(row_cells[1].prop("class") in temp_status , "InfoTable.status.2",
+                            msg="unknown bugstatus '%s' in InfoTable.parse()" %row_cells[1].prop("class"),
+                            error_type=exceptions.VALUEERROR,
+                            url=self.__url)
             status = temp_status[row_cells[1].prop("class")]
             
-            assert row_cells[2], "Wrong XPath-Expr in InfoTable.parse() 'importance' (%s)" %(DEBUG_URL or "unknown")
-            assert row_cells[2].prop("class") in temp_importance, "unknown bugimportance '%s' in InfoTable.parse() (%s)" %(row_cells[2].prop("class"), DEBUG_URL or "unknown")
+            parse_error(row_cells[2], "InfoTable.importance.1", xml=row_cells, url=self.__url)
+            parse_error(row_cells[2].prop("class") in temp_importance, "InfoTable.importance.2",
+                            msg="unknown bugimportance '%s' in InfoTable.parse()" %row_cells[2].prop("class"),
+                            error_type=exceptions.VALUEERROR,
+                            url=self.__url)
             importance = temp_importance[row_cells[2].prop("class")]
             
             assignee = row_cells[3].xpathEval("a")
@@ -392,6 +421,39 @@ class InfoTable(LPTasks):
 
         self.parsed = True
         return True
+        
+    def _LP_create_task(self, task, force_changes, ignore_lp_errors):
+        tsk = task.component._type or ""
+        if tsk.endswith(".product"):
+            url = "%s/+choose-affected-product" %self._url
+            args = {"field.visited_steps": "choose_product, specify_remote_bug_url",
+                    "field.product": str(task.component.affects),
+                    "field.actions.continue": "Add to Bug Report"}
+            result = self._connection.post(url, args)
+            if result.url == url:
+                # possible errors:
+                #  * project does not exist
+                #  * 'A fix for this bug has already been requested for ...'
+                raise exceptions.choose_pylpbugsError(error_type=exceptions.VALUEERROR, text=result.text, url=url)
+        elif tsk.endswith(".sourcepackagename"):
+            url = "%s/+distrotask" %self._url
+            args = { "field.distribution": task.component.target or "ubuntu",
+                     "field.distribution-empty-marker": 1,
+                     "field.sourcepackagename": task.component.sourcepackage or "",
+                     "field.visited_steps": "specify_remote_bug_url",
+                     "field.bug_url": task.component.remote or "",
+                     "field.actions.continue": "Continue"}
+            result = self._connection.post(url, args)
+            if result.url == url:
+                # possible errors
+                # * "This bug is already open on <distro> with no package specified."
+                # * "There is no package in <distro> named <sourcepackage>"
+                # (not sure of other errors)
+                raise exceptions.choose_pylpbugsError(error_type=exceptions.VALUEERROR, text=result.text, url=url)
+                
+        else:
+            raise NotImplementedError
+        
 
    
 class BugReport(object):
@@ -422,12 +484,13 @@ class BugReport(object):
         return "<BugReport>"
 
     def parse(self):
-        assert not self.parsed, "BugReport.parsed() should never excecuted twice" # TODO: add this to each parse()
+        if self.parsed:
+            return True
         
         # getting text (description) of the bugreport
         description = self.__xml.xpathEval('//body//div[@class="report"]/\
 div[@id="bug-description"]')
-        assert description, "Wrong XPath-Expr in BugReport.parse() 'description' (%s)" %(DEBUG_URL or "unknown")
+        parse_error(description, "BugReport.description", xml=self.__xml, url=self.__url)
         
         # hack to change </p> into \n - any better idea? - NOT just eye-candy
         # this is also the format needed when committing changes
@@ -440,10 +503,10 @@ div[@id="bug-description"]')
         
         # getting tile (summary) of the bugreport
         title = self.__xml.xpathEval('//title')
-        assert title, "Wrong XPath-Expr in BugReport.parse() 'title' (%s)" %(DEBUG_URL or "unknown")
+        parse_error(title, "BugReport.title", self.__xml, self.__url)
         titleFilter = 'Bug #[0-9]* in ([^:]*?): (.*)'
         title = re.findall(titleFilter, title[0].content)
-        assert title, "Wrong RegEx in BugReport.parse() 'title' (%s)" %(DEBUG_URL or "unknown")
+        parse_error(title, "BugReport.__title", url=self.__url)
         self.__title = title[0][1].rstrip('\xe2\x80\x9d').lstrip('\xe2\x80\x9c')
         
         # getting target and sourcepackage
@@ -456,22 +519,31 @@ div[@id="bug-description"]')
             self.sourcepackage = None
         
         # getting tags of bugreport
-        tags = self.__xml.xpathEval('//body//div[@class="report"]/\
-div[@id="bug-tags"]//a/text()')
+        tags = self.__xml.xpathEval('//body//div[@class="report"]//\
+div[@id="bug-tags"]//a')
         self.__tags = [i.content for i in tags]
 
-        m = self.__xml.xpathEval('//div[@id="maincontent"]//div[@style="float: left;"]')
-        assert m, "Wrong XPath-Expr in BugReport.parse() 'info line' (%s)" %(DEBUG_URL or "unknown")
+        
+        # (thekorn) 20080828: design on edge changed
+        # edge and stable have significantly diverged
+        # if they are in sync again this conditional construct can
+        # be removed again
+        # affected: nickname, date, reporter
+        m = self.__xml.xpathEval('//span[@class="object identifier"]') #stable
+        m = m or self.__xml.xpathEval('//div[@class="object identifier"]') #edge
+        parse_error(m, "BugReport.__nickname", xml=self.__xml, url=self.__url)
         r = re.search(r'\(([^\)]+)\)', m[0].content)
         self.__nickname = (r and r.group(1)) or None
         
-        d = m[0].xpathEval('span')
-        assert d, "Wrong XPath-Expr in BugReport.parse() 'date' (%s)" %(DEBUG_URL or "unknown")
+        d = self.__xml.xpathEval('//span[@class="object timestamp"]/span') #stable
+        d = d or self.__xml.xpathEval('//p[@class="object timestamp"]/span') #edge
+        parse_error(d, "BugReport.date", xml=m[0], url=self.__url)
         self.date = LPTime(d[0].prop("title"))
         
-        d = m[0].xpathEval('a')
-        assert d, "Wrong XPath-Expr in BugReport.parse() 'reporter' (%s)" %(DEBUG_URL or "unknown")
         
+        d = self.__xml.xpathEval('//span[@class="object timestamp"]/a') #stable
+        d = d or self.__xml.xpathEval('//p[@class="object timestamp"]/a') #edge
+        parse_error(d, "BugReport.reporter", xml=m[0], url=self.__url)
         self.reporter = user.parse_html_user(d[0])
         
         self.__cache = {"title": self.__title, "description" : self.__description,
@@ -521,7 +593,7 @@ div[@id="bug-tags"]//a/text()')
             result = self.__connection.get(url)
             xmldoc = libxml2.htmlParseDoc(unicode_for_libxml2(result.text), "UTF-8")
             x = xmldoc.xpathEval('//textarea[@name="field.description"]')
-            assert x, "Wrong XPath-Expr in BugReport.commit() 'description' (%s)" %(DEBUG_URL or "unknown")
+            parse_error(x, "BugReport.description_raw", xml=xmldoc, url=url)
             self.__description_raw = x[0].content
         return self.__description_raw
         
@@ -546,7 +618,9 @@ div[@id="bug-tags"]//a/text()')
                 nickname = ""
             else:
                 nickname = self.nickname
-            assert self.title and description, ".title and .description needed in BugReport.commit() (%s)" %(DEBUG_URL or "unknown")
+            if not (self.title and description):
+                raise exceptions.PythonLaunchpadBugsValueError(msg="To change a bugreport 'description' \
+and 'title' don't have to be empty", url=self.__url)
             args = { 'field.actions.change': '1', 
                  'field.title': self.title, 
                  'field.description': description, 
@@ -589,9 +663,10 @@ class Attachment(LPAttachment):
     TODO: work on docstring
     """
     def __init__(self, connection, url=None, localfilename=None, localfileobject=None,
-                        description=None, is_patch=None, contenttype=None):
+                        description=None, is_patch=None, contenttype=None, comment=None):
         LPAttachment.__init__(self, connection, url, localfilename,
-                                localfileobject, description, is_patch, contenttype)
+                                localfileobject, description, is_patch,
+                                contenttype, comment)
             
     def get_bugnumber(self):
         if self.is_up:
@@ -603,7 +678,7 @@ class Attachment(LPAttachment):
             
     def get_edit_url(self):
         if self.is_up:
-            return utils.valid_lp_url(self._edit, utils.BUG)
+            return valid_lp_url(self._edit, BASEURL.BUG)
         
 
 class Attachments(LPAttachments):
@@ -625,12 +700,16 @@ class Attachments(LPAttachments):
             for i in self:
                 i._edit = all_att.get(i.url, None)
                 if not i._edit and i.is_up:
-                    raise RuntimeError, "this should never happen"
+                    parse_error(False, "Attachments.edit.1",
+                                    msg="There is an attachment (id=%s) which is added to a comment but does not appear in the sidepanel ('%s')" %(i.id, self.__comments._url),
+                                    error_type=exceptions.RUNTIMEERROR)
         else:
-            assert not self._current, "There seems to be a file added to a comment, but no enty in the side-panel ('%s')" %self.__comments._url
+            parse_error(not self._current, "Attachments.edit.2",
+                            msg="Unable to parse the 'attachments' sidepanel although there are files attached to comments ('%s')" %self.__comments._url,
+                            error_type=exceptions.RUNTIMEERROR)
             
                 
-    def commit(self, force_changes=True, ignore_lp_errors=False):
+    def commit(self, force_changes=True, ignore_lp_errors=False, com_subject=None):
         """
             when adding a new attachment, this attachment is added as a new comment.
             this new comment has no subject but a subject is required.
@@ -654,7 +733,7 @@ class Attachments(LPAttachments):
             assert isinstance(attachment, Attachment), "<attachment> has to be an instance of 'Attachment'"
             c = Comment(attachment=(attachment,))
             # delegate to Comments
-            self.__comments._lp_add_comment(c, True, False)
+            self.__comments._lp_add_comment(c, force_changes, ignore_lp_errors, com_subject)
             
         changed = set(self.changed)
         for i in changed:
@@ -682,28 +761,32 @@ class Comments(LPComments):
         self.__url = url
         
     def parse(self):
-        url = self.__url or "unknown"
         for com in self.__xml:
             m = com.xpathEval('div[@class="boardCommentDetails"]/a[1]')
-            assert m, "Wrong XPath-Expr in Comments.parse() 'user' (%s)" %url
+            parse_error(m, "Comments.user", xml=self.__xml, url=self.__url)
             com_user = user.parse_html_user(m[0])
 
             m = com.xpathEval('div[@class="boardCommentDetails"]/a[2]')
-            assert m, "Wrong XPath-Expr in Comments.parse() 'nr' (%s)" %url
+            parse_error(m, "Comments.nr", xml=self.__xml, url=self.__url)
             com_nr = m[0].prop('href').split('/')[-1]
 
             m = com.xpathEval('div[@class="boardCommentDetails"]/span')
-            assert m, "Wrong XPath-Expr in Comments.parse() 'date' (%s)" %url
+            parse_error(m, "Comments.date", xml=self.__xml, url=self.__url)
             com_date = LPTime(m[0].prop('title'))
             
             m = com.xpathEval('div[@class="boardCommentBody"]/div')
-            assert m, "Wrong XPath-Expr in Comments.parse() 'text' (%s)" %url
+            parse_error(m, "Comments.text", xml=self.__xml, url=self.__url)
             com_text = m[0].content
             
             com_attachments = set()
-            m = com.xpathEval('div[@class="boardCommentBody"]/ul/li/a')
+            m = com.xpathEval('div[@class="boardCommentBody"]/ul/li')
             for a in m:
-                com_attachments.add(Attachment(self.__connection, url=a.prop('href'), description=a.content))
+                a_url = a.xpathEval("a").pop()
+                a = re.search(r",\n +(\S+)(;|\))", a.content)
+                parse_error(a, "Comments.attachment.re.%s" %a_url.prop('href'), xml=com, url=self.__url)
+                a_contenttype = a.group(1)
+                com_attachments.add(Attachment(self.__connection, url=a_url.prop('href'),
+                    description=a_url.content, comment=com_nr, contenttype=a_contenttype))
                 
             c = Comment(text=com_text, attachment=com_attachments)
             c.set_attr(nr=com_nr,user=com_user,date=com_date)
@@ -720,10 +803,10 @@ class Comments(LPComments):
         return self.__url
         
                 
-    def _lp_add_comment(self, comment, force_changes, ignore_lp_errors):
+    def _lp_add_comment(self, comment, force_changes, ignore_lp_errors, com_subject):
         assert isinstance(comment, Comment)
         args = {
-            'field.subject': comment.subject or "",
+            'field.subject': comment.subject or com_subject or "Re:",
             'field.comment': comment.text or "", 
             'field.actions.save': '1',
             'field.filecontent.used': '',
@@ -763,10 +846,10 @@ class Comments(LPComments):
                     raise ValueError, "launchpad.net error: %s" %y[0].content
         return result
             
-    def commit(self, force_changes=False, ignore_lp_errors=True):
+    def commit(self, force_changes=False, ignore_lp_errors=True, com_subject=None):
         for i in self.changed:
             if i.action == "added":
-                self._lp_add_comment(i.component, force_changes, ignore_lp_errors)
+                self._lp_add_comment(i.component, force_changes, ignore_lp_errors, com_subject)
             else:
                 raise AttributeError, "Unknown action '%s' in Comments.commit()" %i.component
 
@@ -784,11 +867,10 @@ class Duplicates(object):
         return "<Duplicates>"
         
     def parse(self):
-        """ bugnumber optional for debugging """
         if self.parsed:
             return True
         # Check if this bug is already marked as a duplicate
-        nodes = self.__xml.xpathEval('//body//div[@id="mainarea"]//p[@class="informational message" and contains(.,"This report is a duplicate of")]/a')
+        nodes = self.__xml.xpathEval('//body//p[@id="duplicate-of"]/a')
         if len(nodes)>0:
             self.__duplicate_of = int(nodes[0].prop('href').split('/').pop())
         result = self.__xml.xpathEval('//body//div[@class="portlet"]/h2[contains(.,"Duplicates of this bug")]/../div[@class="portletBody"]/div/ul//li/a')
@@ -850,11 +932,21 @@ class Secrecy(object):
     def parse(self):
         if self.parsed:
             return True
-        assert self.__xml, "Wrong XPath-Expr in Secrecy.parse() '__xml' (%s)" %(DEBUG_URL or "unknown")
-        if self.__xml[0].xpathEval('img[@alt="(Security vulnerability)"]'):
-            self.__security = True
-        if self.__xml[0].xpathEval('img[@alt="(Private)"]'):
-            self.__private = True
+        # (thekorn) 20080711: design on edge changed
+        # edge and stable have significantly diverged
+        # if they are in sync again this conditional construct can
+        # be removed again
+        #~ this error check makes no sense
+        #~ parse_error(self.__xml, "Secrecy.__xml", xml=self.__xml, url=self.__url)
+        stable_xml = self.__xml.xpathEval('//body//div[@id="big-badges"]')
+        if stable_xml:
+            if stable_xml[0].xpathEval('img[@alt="(Security vulnerability)"]'):
+                self.__security = True
+            if stable_xml[0].xpathEval('img[@alt="(Private)"]'):
+                self.__private = True
+        else:
+            self.__private = bool(self.__xml.xpathEval('//a[contains(@href, "+secrecy")]/strong'))
+            self.__security = bool(self.__xml.xpathEval('//div[contains(@style, "/@@/security")]'))
         self.__cache = {"security": self.__security, "private" : self.__private}
         self.parsed = True
         return True
@@ -917,22 +1009,21 @@ class Subscribers(LPSubscribers):
         
         if self.parsed:
             return True
-        assert self.__xml, "Wrong XPath-Expr in Subscribers.parse() '__xml' (%s)" %(DEBUG_URL or "unknown")
-        for k in self.__xml:
-            if k.prop("class") == "person":
-                x = k.xpathEval('../b')
-                if x:
-                    if x[0].content == "From duplicates:":
-                        attr = "duplicates"
-                    elif x[0].content == "Also notified:":
-                        attr = "notified"
-                    else:
-                        raise AssertionError, "Subscribers.parse(): unknown category %s" %x[0].content
-                else:
-                    attr = "directly"
-                nodes = k.xpathEval("li/a")
+        parse_error(self.__xml, "Subscribers.__xml", xml=self.__xml, url=self.__url)
+        xml = self.__xml[0].xpathEval("section[@id]")
+        xml = xml or self.__xml[0].xpathEval('div[(@class="section" or @class="Section") and @id]') #fix for v6739
+        if xml:
+            sections_map = {"subscribers-direct": "directly",
+                            "subscribers-indirect": "notified",
+                            "subscribers-from-duplicates": "duplicates",
+                            }
+            for s in xml:
+                kind = sections_map[s.prop("id")]
+                nodes = s.xpathEval("div/a")
                 for i in nodes:
-                    self[attr].add(user.parse_html_user(i))
+                    self[kind].add(user.parse_html_user(i))
+        else:
+            parse_error(False, "Subscribers.__xml.edge.stable", xml=self.__xml, url=self.__url)
         self.parsed = True
         return True
                 
@@ -970,17 +1061,18 @@ class Subscribers(LPSubscribers):
         
 
 class ActivityWhat(str):
-    def __new__(cls, what):
+    def __new__(cls, what, url=None):
         obj = super(ActivityWhat, cls).__new__(ActivityWhat, what)
         obj.__task = None
         obj.__attribute = None
         x = what.split(":")
-        assert len(x) < 3, "Wrong syntax in ActivityLog.ActivityWhat (%s)" %(DEBUG_URL or "unknown")
         if len(x) == 2:
             obj.__task = x[0]
             obj.__attribute = x[1].strip()
-        else:
+        elif len(x) == 1:
             obj.__attribute = x[0]
+        else:
+            raise ValueError
         return obj
         
     @property
@@ -1077,18 +1169,24 @@ class ActivityLog(object):
         
         self.__xmldoc = libxml2.htmlParseDoc(unicode_for_libxml2(page.text), "UTF-8")
         table = self.__xmldoc.xpathEval('//body//table[@class="listing"][1]//tbody//tr')
+        parse_error(table, "ActivityLog.__table", xml=self.__xmldoc, url=self.__url)
         for row in table:
             r = row.xpathEval("td")
-            assert len(r) == 6, "Wrong XPath-Expr in ActivityLog.parse() 'r('td')' (%s)" %(DEBUG_URL or "unknown")
+            parse_error(len(r) == 6, "ActivityLog.len(td)", xml=row, url=self.__url)
+            
             date = LPTime(r[0].content)
             x = r[1].xpathEval("a")
+            parse_error(x, "ActivityLog.lp_user", xml=row, url=self.__url)
             lp_user = user.parse_html_user(x[0])
-            what = r[2].content
+            try:
+                what = ActivityWhat(r[2].content)
+            except ValueError:
+                parse_error(False, "ActivityLog.ActivityWhat", xml=self.__xmldoc, url="%s/+activity" %self.__url)
             old_value = r[3].content
             new_value = r[4].content
             message = r[5].content
             
-            self.__activity.append(Activity(date, lp_user, ActivityWhat(what),
+            self.__activity.append(Activity(date, lp_user, what,
                                         old_value, new_value, message))
         self.__activity_rev = self.__activity[::-1]
         self.parsed = True
@@ -1216,10 +1314,6 @@ class Bug(BugBase):
             
         BugBase.__init__(self, bug, url, connection)
 
-        if DEBUG == True:
-            global DEBUG_URL
-            DEBUG_URL = self.__url
-
         bugpage = self.__connection.get(self.__url)
 
         # self.text contains the whole HTML-formated Bug-Page
@@ -1232,11 +1326,11 @@ class Bug(BugBase):
         
         self.__bugreport = BugReport(connection=self.__connection, xml=self.xmldoc, url=self.__url)
         self.__infotable = InfoTable(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//table[@class="listing" or @class="duplicate listing"][1]'), url=self.__url)
-        self.__comments = Comments(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//div[@class="boardComment "]'), url=self.__url)
+        self.__comments = Comments(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//div[normalize-space(@class)="boardComment"]'), url=self.__url)
         self.__attachments = Attachments(self.__comments, self.__connection, self.xmldoc.xpathEval('//body//div[@id="portlet-attachments"]/div/div/ul'))
         self.__duplicates = Duplicates(connection=self.__connection, xml=self.xmldoc, url=self.__url)
-        self.__secrecy = Secrecy(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//div[@id="big-badges"]'), url=self.__url)
-        self.__subscribers = Subscribers(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//div[@id="portlet-subscribers"]//div/ul'), url=self.__url)
+        self.__secrecy = Secrecy(connection=self.__connection, xml=self.xmldoc, url=self.__url)
+        self.__subscribers = Subscribers(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//div[@id="portlet-subscribers"]'), url=self.__url)
         self.__mentor = Mentoring(connection=self.__connection, xml=self.xmldoc.xpathEval('//body//img [@src="/@@/mentoring"]/parent::p//a'), url=self.__url)
         self.__activity = ActivityLog(connection=self.__connection, url=self.__url)
         self.__branches = Branches(self.__connection, self.xmldoc.xpathEval('//body//div[@class="bug-branch-summary"]'), self.__url)
@@ -1265,7 +1359,10 @@ class Bug(BugBase):
         
     def commit(self, force_changes=False, ignore_lp_errors=True):
         for i in self.changed:
-            result = i.component.commit(force_changes, ignore_lp_errors)
+            if isinstance(i.component, Comments):
+                result = i.component.commit(force_changes, ignore_lp_errors, "Re: %s" %self.title)
+            else:
+                result = i.component.commit(force_changes, ignore_lp_errors)
                         
     def revert(self):
         """ need a function to revert changes """
@@ -1350,10 +1447,15 @@ class Bug(BugBase):
     # subscription
     
     get_subscriptions = _gen_getter("__subscribers")
-    get_comments = _gen_getter("__comments")
     get_attachments = _gen_getter("__attachments")    
+    def get_comments(self):
+        x = self.attachments
+        if not self.__comments.parsed:
+            self.comments.parse()
+        return self.__comments
 
     def get_subscriptions_category(self, type):
+        """ get subscriptions for a given category, possible categories are "directly", "notified", "duplicates" """
         if not self.__subscribers.parsed:
             self.__subscribers.parse()
         return self.__subscribers.get_subscriptions(type)
@@ -1382,9 +1484,27 @@ def create_new_bugreport(product, summary, description, connection, tags=[], sec
     else:
         url = "https://bugs.launchpad.net/%s/+filebug-advanced" %product["name"]
     
-    result = connection.post(url, args)
+    try:
+        result = connection.post(url, args)
+    except exceptions.LaunchpadError, e:
+        try:
+            x = connection.needs_login()
+        except exceptions.LaunchpadError:
+            x = False
+        if x:
+            raise exceptions.LaunchpadLoginError(url)
+        elif isinstance(e, exceptions.LaunchpadURLError):
+            if "Page not found" in e.msg:
+                m = """Maybe there is no product '%s' in """ %product["name"]
+                if product.has_key("target"):
+                    m += """the distribution '%s'""" %product["target"]
+                else:
+                    m += "launchpad.net"
+                raise exceptions.PythonLaunchpadBugsValueError(msg=m)
+        else:
+            raise
+            
     if not result.url.endswith("+filebug-advanced"):
         return Bug(url=result.url, connection=connection)
     else:
-        raise ValueError, "There is a problem with the information you entered. \
- Please fix it and try again.\nurl: %s\nargs: %s" %(url, args)
+        raise exceptions.choose_pylpbugsError(error_type=exceptions.VALUEERROR, text=result.text, url=url)
