@@ -1,8 +1,6 @@
 """
 TODO:
- * move user/Comment-class to bugbase (?) as this class is used in html_bug and text_bug
  * check .private/.security: this seems to be wrong/maybe it  does not work
- 
  * remove these ugly dict-element vs. object-attribute things
 """
 
@@ -10,33 +8,30 @@ from bugbase import Bug as BugBase
 import email
 import cStringIO as StringIO
 import re
+import ConfigParser
+
 from lptime import LPTime
 from lphelper import user, product
 from tasksbase import LPTasks, LPTask
 from attachmentsbase import LPAttachments, LPAttachment
 from commentsbase import LPComments, LPComment
-
-
-#from html_bug import Bug as BugBase # using html_bug as fallback
         
 class Task(LPTask):
-    def __init__(self, value_dict):
-        #unpack values
-        task_dict = {}
-        for k, i in value_dict.iteritems():
-            if i:
-                task_dict[k] = i.pop()
+    def __init__(self, task_dict):
                 
         #print task_dict
         task_dict["affects"], task_dict["targeted_to"] = product.parse_text_product(task_dict["task"])
-        for i in ["reporter", "assignee"]:
-            if task_dict.has_key(i):            
-                task_dict[i] = user.parse_text_user(task_dict[i])
-        for i in ["date-created", "date-confirmed", "date-assigned", "date-inprogress", "date-closed"]:
+        for i in ("reporter", "assignee"):
+            task_dict[i] = user.parse_text_user(task_dict[i])
+        for i in ("date-created", "date-confirmed", "date-assigned",
+                  "date-inprogress", "date-closed", "date-left-new",
+                  "date-incomplete", "date-triaged", "date-fix-committed",
+                  "date-fix-released"):
             if task_dict.has_key(i):
                 task_dict[i] = LPTime(task_dict[i])
-        if task_dict.has_key("watch"):
-            task_dict["remote"] = True
+            else:
+                task_dict[i] = None
+        task_dict["remote"] = task_dict.get("watch", None)
         #print task_dict
         LPTask.__init__(self, task_dict)
                     
@@ -54,28 +49,51 @@ class Task(LPTask):
     
     def get_date_closed(self):
         return self._date_closed
+        
+    def get_date_left_new(self):
+        return self._date_left_new
+        
+    def get_date_incomplete(self):
+        return self._date_incomplete
+
+    def get_date_triaged(self):
+        return self._date_triaged
+
+    def get_date_fix_committed(self):
+        return self._date_fix_committed
+
+    def get_date_fix_released(self):
+        return self._date_fix_released
     
     def get_user(self):
         return self._user
+
+    def get_component(self):
+        return self._component
         
         
 class BugReport(object): 
     def __init__(self, bug_dict, connection=None):
         self.__data = bug_dict
         for i in ("tags", "duplicates"):
-            self.__data[i] = self.__data.get(i, [])
-            if self.__data[i]:
-                self.__data[i] = self.__data[i][0].split()
-        self.__data["bugnumber"] = int(bug_dict["bug"][0])
-        self.__data["reporter"] = [user.parse_text_user(u) for u in self.__data.get("reporter", [])]
-        self.__data["subscribers"] = [user.parse_text_user(u) for u in self.__data.get("subscribers", [])]
+            self.__data[i] = self.__data[i].split()
+        self.__data["bugnumber"] = int(bug_dict["bug"])
+        del self.__data["bug"]
+        self.__data["reporter"] = user.parse_text_user(self.__data["reporter"])
+        self.__data["subscribers"] = [user.parse_text_user(u) for u in self.__data["subscribers"].split("\n") if u]
+        
         self.__data["description"] = ""
-        for i in ["date-reported", "date-updated"]:
-            if self.__data.has_key(i):
-                self.__data[i] = LPTime(self.__data[i][0])
+        for i in ("date-reported", "date-updated"):
+            self.__data[i] = LPTime(self.__data[i])
         a = list()
-        for i in self.__data.get("attachments",[]):
-            url, contenttype = i.split(" ", 1)
+        for i in [i for i in self.__data["attachments"].split("\n") if i]:
+            try:
+                url, contenttype = i.split(" ", 1)
+            except ValueError:
+                if not i.startswith("http://"):
+                    continue
+                else:
+                    raise
             contenttype = contenttype.split(";")[0]
             a.append(Attachment(url, contenttype, connection, self.__data["bugnumber"]))
         self.__data["attachments"] = Attachments(a)
@@ -144,84 +162,46 @@ class TextPage(object):
         
     def parse(self):
         text = self.__connection.get(self.__url).text
-        text_file = StringIO.StringIO(text)
-        end_start = False
-        k = None
-        v = []
-        block = None
-        task_count = 0
+        parts = text.split("\n\n")
         tmp_task = LPTasks({"url": self.__url})
+        self.__data["comments"] = Comments()
         current_task_tupel = LPTasks.current_from_url(self.__url)
-        cur_data = {}
-        for l in text_file:
-            if l == "\n":
-                if block == "task":
-                    if k:
-                        cur_data[k] = v
-                    task_obj = Task(cur_data)
-                    if task_obj.is_current(current_task_tupel):
-                        tmp_task._current = len(tmp_task)
-                    tmp_task.append(task_obj)
-                elif block == "bug":
-                    self.__data["bug"] = BugReport(cur_data, connection=self.__connection)
-                else:
-                    self.__data[block][k] = v
-                block = None
-                cur_data = {}
-                k = None
-                v = []
+        while parts:
+            #~ print "LEN", len(parts)
+            if parts[0].startswith("bug:"):
+                x = parts.pop(0)
+                x = "[bug]\n" + x
+                #~ print "BUG"
+                data = ConfigParser.ConfigParser()
+                data.readfp(StringIO.StringIO(x))
+                self.__data["bug"] = BugReport(dict(data.items("bug")), connection=self.__connection)
+            elif parts[0].startswith("task:"):
+                x = parts.pop(0)
+                x = "[task]\n" + x
+                #~ print "TASK"
+                data = ConfigParser.ConfigParser()
+                data.readfp(StringIO.StringIO(x))
+                task_obj = Task(dict(data.items("task")))
+                if task_obj.is_current(current_task_tupel):
+                    tmp_task._current = len(tmp_task)
+                tmp_task.append(task_obj)
+                continue
+            elif parts[0].startswith("Content-Type:"):
+                l = "\n\n".join(parts)
+                comments_msg = email.message_from_string(l)
+                messages = comments_msg.get_payload()
+                description = messages.pop(0)
+                self.__data["bug"]._add_description(description.get_payload(decode=True))
+                for i, msg in enumerate(messages):
+                    self.__data["comments"].append(Comment(i+1, msg.get_all("Author"), msg.get_all("Date"), msg.get_payload(decode=True)))
+                parts = False
+                continue
             else:
-                if l.startswith(" "):
-                    if block == "bug":
-                        try:
-                            cur_data[k].append(l.strip(" \n"))
-                        except KeyError:
-                            cur_data[k] = [l.strip(" \n")]
-                    else:
-                        v.append(l.strip(" \n"))
-                else:
-                    x = l.split(":",1)
-                    if l.startswith("Content-Type:"):
-                        end_start = True
-                        break
-                    assert len(x) == 2, "wrong line:%s" %l
-                    if not block:
-                        block = x[0]
-                        if block == "task" or block == "bug":
-                            cur_data = {}
-                        else:
-                            self.__data[block] = {}
-                    if k and v:
-                        if block == "task" or block == "bug":
-                            cur_data[k] = v
-                        else:
-                            self.__data[block][k] = v
-                        k = None
-                        v = []
-                    k = x[0]
-                    y = x[1].strip(" \n")
-                    if y:
-                        v = [y]
-        if block:
-            if block == "task":
-                tmp_task.append(Task(cur_data))
-            elif block == "bug":
-                self.__data["bug"] = BugReport(cur_data)
-            else:
-                self.__data[block][k] = v
-        if tmp_task._current == None and len(tmp_task) == 1:
+                raise RuntimeError
+        if tmp_task._current is None and len(tmp_task) == 1:
             # "only one task, this must be current"
             tmp_task._current = 0
         self.__data["task"] = tmp_task
-        self.__data["comments"] = Comments()
-        if end_start:
-            l += text_file.read()
-            comments_msg = email.message_from_string(l)
-            messages = comments_msg.get_payload()
-            description = messages.pop(0)
-            self.__data["bug"]._add_description(description.get_payload(decode=True))
-            for i, msg in enumerate(messages):
-                self.__data["comments"].append(Comment(i+1, msg.get_all("Author"), msg.get_all("Date"), msg.get_payload(decode=True)))
         self.parsed = True
                     
 
@@ -240,10 +220,10 @@ class Bug(BugBase):
         return self.__textpage["bug"].bugnumber[0]
         
     def get_reporter(self):
-        return self.__textpage["bug"].reporter[0]
+        return self.__textpage["bug"].reporter
         
     def get_title(self):
-        return self.__textpage["bug"].title[0]
+        return self.__textpage["bug"].title
         
     def get_subscriptions(self):
         return set(self.__textpage["bug"].subscribers)
@@ -284,18 +264,10 @@ class Bug(BugBase):
         return self.__textpage["task"].current.affects
 
     def get_private(self):
-        a = self.__textpage["private"]
-        try:
-            return a[0]
-        except IndexError:
-            return False
+        return bool(self.__textpage["bug"].private)
 
     def get_security(self):
-        a = self.__textpage["security"]
-        try:
-            return a[0]
-        except IndexError:
-            return False
+        return bool(self.__textpage["bug"].security)
 
     def get_tags(self):
         return self.__textpage["bug"].tags
